var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import 'reflect-metadata';
import { DOM } from 'aurelia-pal';
var patches = newPatches();
var aureliaDialogWarningMessage = 'aurelia-dialog causes Cypress to crash. Removing plugin...';
var stylesCache = new Map();
function copyStyles(componentName) {
    var hash = componentName;
    var styles = document.head.querySelectorAll('style');
    if (styles.length) {
        console.log('Injected %d styles into view', styles.length);
        stylesCache.set(hash, styles);
    }
    else {
        console.log('No styles injected for this component, checking cache...');
        var cachedStyles = stylesCache.get(hash);
        if (cachedStyles) {
            styles = cachedStyles;
        }
    }
    if (!styles) {
        return;
    }
    var parentDocument = window.parent.document;
    var projectName = Cypress.config('projectName');
    var appIframeId = "Your App: '" + projectName + "'";
    var appIframe = parentDocument.getElementById(appIframeId);
    var head = appIframe.contentDocument.querySelector('head');
    styles.forEach(function (style) {
        head.appendChild(style);
    });
}
function walkNode(node, func) {
    if (!node ||
        !node.firstChild) {
        return;
    }
    func(node);
    node = node.firstChild;
    while (node) {
        walkNode(node, func);
        node = node.nextSibling;
    }
}
function newPatches() {
    return {
        aureliaDialogDisabled: false
    };
}
var _ = (function () {
    function _() {
    }
    _.Patches = function () {
        return __assign({}, patches);
    };
    return _;
}());
export { _ };
var StageComponent = (function () {
    function StageComponent() {
    }
    StageComponent.withResources = function (resources) {
        if (resources === void 0) { resources = []; }
        var doc = cy.state('document');
        while (doc.body.firstChild) {
            doc.body.removeChild(doc.body.firstChild);
        }
        return new ComponentTester().withResources(resources);
    };
    return StageComponent;
}());
export { StageComponent };
var ComponentTester = (function () {
    function ComponentTester() {
        this.html = '';
        this.resources = [];
        this.bindingContext = {};
        this.configure = function (aurelia) {
            return aurelia.use.standardConfiguration();
        };
    }
    ComponentTester.prototype.withResources = function (resources) {
        if (typeof resources === 'string') {
            this.resources = [resources];
            return this;
        }
        this.resources = resources;
        return this;
    };
    ComponentTester.prototype.inView = function (html) {
        this.html = html;
        return this;
    };
    ComponentTester.prototype.boundTo = function (bindingContext) {
        this.bindingContext = bindingContext;
        return this;
    };
    ComponentTester.prototype.bootstrap = function (configure) {
        this.configure = configure;
        return this;
    };
    ComponentTester.prototype.create = function (bootstrap) {
        var _this = this;
        patches = newPatches();
        return new Cypress.Promise(function (resolve, reject) {
            var doc = cy.state('document');
            return bootstrap(function (aurelia) {
                return Promise.resolve(_this.configure(aurelia)).then(function () {
                    if (_this.resources) {
                        aurelia.use.globalResources(_this.resources);
                    }
                    _this.viewModel = undefined;
                    aurelia.container.registerInstance(Element, doc.createElement);
                    aurelia.container.registerInstance(DOM.Element, doc.createElement);
                    {
                        var plugins = aurelia.use.info;
                        if (plugins) {
                            plugins = plugins.filter(function (plugin) {
                                if (plugin &&
                                    plugin.moduleId &&
                                    plugin.moduleId === 'aurelia-dialog') {
                                    console.warn(aureliaDialogWarningMessage);
                                    patches.aureliaDialogDisabled = true;
                                    return false;
                                }
                                return true;
                            });
                            aurelia.use.info = plugins;
                        }
                    }
                    return aurelia.start().then(function () {
                        if (doc.activeElement !== doc.body) {
                            doc.body.focus();
                        }
                        while (doc.body.firstChild) {
                            doc.body.removeChild(doc.body.firstChild);
                        }
                        doc.body.innerHTML = _this.html;
                        var bindingContext = _this.bindingContext;
                        var rootElement = doc.body;
                        return aurelia.enhance(bindingContext, rootElement).then(function () {
                            copyStyles(_this.resources.join(','));
                            walkNode(rootElement, function (el) {
                                if (!_this.viewModel &&
                                    el.au) {
                                    var controllers = el.au;
                                    var controller = void 0;
                                    for (var key in controllers) {
                                        if (!controllers.hasOwnProperty(key)) {
                                            continue;
                                        }
                                        controller = controllers[key];
                                        break;
                                    }
                                    if (controller &&
                                        controller.view) {
                                        var viewUrl = controller.view.resources.viewUrl;
                                        var viewUrlNoExt = viewUrl.split('.').slice(0, -1).join('.');
                                        if (_this.resources.indexOf(viewUrl) > -1 ||
                                            _this.resources.indexOf(viewUrlNoExt) > -1) {
                                            _this.viewModel = controller.viewModel;
                                        }
                                    }
                                }
                            });
                            if (!_this.viewModel) {
                                console.warn('Unable to determine viewModel for mounted component. This is expected behaviour for @containerless components.');
                            }
                            window.requestAnimationFrame(function () {
                                window.requestAnimationFrame(function () {
                                    window.requestAnimationFrame(function () {
                                        window.requestAnimationFrame(function () {
                                            resolve();
                                        });
                                    });
                                });
                            });
                        });
                    }).catch(function (err) {
                        reject(err);
                        return;
                    });
                });
            });
        });
    };
    return ComponentTester;
}());
export { ComponentTester };
